#!/bin/bash 
#
# Helix P4D maintenance script
#
# This script creates a checkpoint of the Perforce Server and
# if successful, cleans up any checkpoints and journals older
# N days, per the configuration settings defined below.
#
# Usage: maintenance.sh servername
#

###############################################################################
# Global Config
CLEANUP_DAYS=14
MINIMUM_JNLS=14
###############################################################################

usage()
{
  cat <<EOS

Usage: maintenance.sh [ [service-name] | --all ]

EOS
}

log()
{
  local OUT=$1
  local LOGFILE=$2
  if [ -t 1 ]; then
    echo $OUT
  fi
  if [ -n "$LOGFILE" ]; then
    echo $(date -Iseconds): $OUT >> $LOGFILE
  fi
}

setup()
{
  local SERVER_NAME=$1
  # First, lets make sure we can see the server (no user switching)
  local out=$(p4dctl env -t p4d $SERVER_NAME P4ROOT 2>&1)

  if echo $out | egrep "^p4dctl (error|warning):" > /dev/null; then
    echo $out
    return 1
  fi
  
  local owner=$(p4dctl list -t p4d 2>/dev/null | awk '{ print $3, $2 }' | egrep "^$SERVER_NAME " | cut -d " " -f2) 
  if ! id $owner >/dev/null ; then
     owner=""
  fi

  # Find last checkpoint?
  local out=$(p4dctl exec -t p4d -f $SERVER_NAME -- "-cshow thisserver" | egrep "^P4ROOT = ")
  if echo $out | egrep "^P4ROOT = " > /dev/null; then
    p4root=$(echo $out | egrep "^P4ROOT = " | cut -d " " -f3)
  else
    echo "Couldn't establish P4ROOT"
    return 1
  fi

  local out=$(p4dctl exec -t p4d -f $SERVER_NAME -- "-cshow thisserver" | egrep "^journalPrefix = ")
  if echo $out | egrep "^journalPrefix = " > /dev/null; then
    p4jnlpre=$p4root/$(echo $out | egrep "^journalPrefix = " | cut -d " " -f3)
    p4jnlpath=$(dirname $p4jnlpre)
    p4jnlpre=$(basename $p4jnlpre)
    jnlprefix=1
  else
    p4jnlpre=$p4root/
    p4jnlpath=$(dirname $p4jnlpre)
    p4jnlpre=$(basename $p4jnlpre)
    jnlprefix=0
  fi

  logfile=$p4jnlpath/maintenance.log
  if [ ! -e $logfile ] && [ -n "$owner" ]; then
    touch $logfile
    chown $owner $logfile
  fi    
}

checkpoint()
{
  local SERVER_NAME=$1
  local p4root=""
  local p4jnlpre=""
  local p4jnlpath=""
  local p4jnlpre=""
  local jnlprefix=0
  local logfile=""

  setup $1 || return 1

  local rjnl=$(p4dctl exec -t p4d -f $SERVER_NAME -- -jd - db.counters | egrep -o "@journal@ @[0-9]+@" | egrep -o "[0-9]+")

  log "Beginning checkpoint of $SERVER_NAME" $logfile

  local out=$(p4dctl checkpoint $SERVER_NAME 2>&1)
  if [ $? -ne 0 ] || echo $out | egrep "^p4dctl (error|warning):" > /dev/null; then
    log "Checkpointing failed!\n$out" $logfile
    return 1
  fi

  local njnl=$(p4dctl exec -t p4d -f $SERVER_NAME -- -jd - db.counters | egrep -o "@journal@ @[0-9]+@" | egrep -o "[0-9]+")

  if [ $jnlprefix == 1 ]; then
    local jnlFile=$p4jnlpath/$p4jnlpre.jnl.$rjnl
    local ckpFile=$p4jnlpath/$p4jnlpre.ckp.$njnl.gz
    local md5File=$p4jnlpath/$p4jnlpre.ckp.$njnl.md5
  else
    local jnlFile=$p4jnlpath/journal.$rjnl
    local ckpFile=$p4jnlpath/checkpoint.$njnl.gz
    local md5File=$p4jnlpath/checkpoint.$njnl.md5
  fi

  local md5Calc=$(zcat $ckpFile | md5sum | egrep -o "[a-fA-F0-9]{32}")
  local md5Test=$(more $md5File | egrep -o "[a-fA-F0-9]{32}")

  log "Checkpoint created: $ckpFile" $logfile

  if [ ${md5Calc,,} == ${md5Test,,} ]; then
    log "Success" $logfile
  else
    log "Checkpoint checksum mismatch!" $logfile
    return 1
  fi

  return 0
}

cleanup()
{
  local SERVER_NAME=$1
  local p4root=""
  local p4jnlpre=""
  local p4jnlpath=""
  local p4jnlpre=""
  local jnlprefix=0
  local logfile=""
  setup $1 || return 1

  log "Beginning cleanup of $SERVER_NAME" $logfile

  # Cleanup time
  local cleanupdays="$CLEANUP_DAYS"

  if [ "$jnlprefix" == 1 ]; then
    local delcount=0
    local allcount=$(find "$p4jnlpath" -name "$p4jnlpre.ckp.*" | egrep "ckp.[0-9]+.gz$" | wc -l)
    local oldcount=$(find "$p4jnlpath" -mtime "+$cleanupdays" -name "$p4jnlpre.ckp.*" | egrep "ckp.[0-9]+.gz$" | wc -l)
    let delcount=allcount-oldcount
    while [ "$oldcount" -gt 0 -a "$delcount" -lt "$MINIMUM_JNLS" ]; do
      let cleanupdays=cleanupdays+1
      oldcount=$(find "$p4jnlpath" -mtime "+$cleanupdays" -name "$p4jnlpre.ckp.*" | egrep "ckp.[0-9]+.gz$" | wc -l)
      let delcount=allcount-oldcount
    done
    if [ "$oldcount" -gt 0 ]; then
      find "$p4jnlpath" -mtime "+$cleanupdays" \( -name "$p4jnlpre.ckp.*" -o -name "$p4jnlpre.jnl.*" \) | xargs rm -f
    fi
  else
    local delcount=0
    local allcount=$(find "$p4jnlpath" -name "checkpoint.*" | egrep "checkpoint.[0-9]+.gz$" | wc -l)
    local oldcount=$(find "$p4jnlpath" -mtime "+$cleanupdays" -name "checkpoint.*" | egrep "checkpoint.[0-9]+.gz$" | wc -l)
    let delcount=allcount-oldcount
    while [ "$oldcount" -gt 0 -a "$delcount" -lt "$MINIMUM_JNLS" ]; do
      let CLEANUP_DAYS=CLEANUP_DAYS+1
      oldcount=$(find "$p4jnlpath" -mtime "+$cleanupdays" -name "checkpoint.*" | egrep "checkpoint.[0-9]+.gz$" | wc -l)
      let delcount=allcount-oldcount
    done
    if [ "$oldcount" -gt 0 ]; then
      find -0 "$p4jnlpath" -mtime "+$cleanupdays" \( -name "checkpoint.*" -o -name "jounal.*" \) | xargs -0 rm -f
    fi
  fi
  log "Complete" $logfile
  return 0
}

if [ $# -ne 1 ]; then
  usage
  exit 1
fi

if [ "$1" == "--all" ]; then
  log "Beginning maintenance"
  i=1
  declare -a SERVERS
  IFS=$"\r\n" GLOBIGNORE='*' command eval "SERVERS=($(p4dctl list -t p4d -EMAINTENANCE=true 2> /dev/null | awk '{print $3}'))"

  for server in ${SERVERS[@]}; do
    checkpoint "$server" && cleanup "$server"
    true
  done
else
  exit checkpoint "$1" && cleanup "$1"
fi

